@@ -20,7 +20,7 @@ Usage: nomad acl policy <subcommand> [options] [args]
   This command groups subcommands for interacting with ACL policies. Nomad's ACL
   system can be used to control access to data and APIs. ACL policies allow a
   set of capabilities or actions to be granted or allowlisted. For a full guide
-  see: https://developer.hashicorp.com/nomad/tutorials/access-control
+  see: https://developer.hashicorp.com/nomad/docs/secure/acl
 
   Create an ACL policy:
 
