@@ -771,6 +771,12 @@ func (a *Agent) finalizeClientConfig(c *clientconfig.Config) error {
 		to configure Nomad to work with Consul.`)
 	}
 
+	// Log deprecation message about setting disk_free_mb
+	if c.DiskFreeMB != 0 {
+		a.logger.Warn(`disk_free_mb is deprecated and ignored by Nomad.
+		Please use client.reserved.disk to configure reservable disk for scheduling.`)
+	}
+
 	// If the operator has not set an intro token via the CLI or an environment
 	// variable, attempt to read the intro token from the file system. This
 	// cannot be used as a CLI override.
