@@ -19,6 +19,7 @@ import (
 	"github.com/gorilla/websocket"
 	"github.com/hashicorp/go-msgpack/v2/codec"
 	cstructs "github.com/hashicorp/nomad/client/structs"
+	"github.com/hashicorp/nomad/helper/pointer"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/plugins/drivers"
 )
@@ -147,18 +148,24 @@ func (s *HTTPServer) allocStop(allocID string, resp http.ResponseWriter, req *ht
 		return nil, CodedError(405, ErrInvalidMethod)
 	}
 
-	noShutdownDelay := false
-	if noShutdownDelayQS := req.URL.Query().Get("no_shutdown_delay"); noShutdownDelayQS != "" {
-		var err error
-		noShutdownDelay, err = strconv.ParseBool(noShutdownDelayQS)
-		if err != nil {
-			return nil, fmt.Errorf("no_shutdown_delay value is not a boolean: %v", err)
-		}
+	noShutdownDelay, err := parseBool(req, "no_shutdown_delay")
+	if err != nil {
+		return nil, err
+	} else if noShutdownDelay == nil {
+		noShutdownDelay = pointer.Of(false)
+	}
+
+	reschedule, err := parseBool(req, "reschedule")
+	if err != nil {
+		return nil, err
+	} else if reschedule == nil {
+		reschedule = pointer.Of(false)
 	}
 
 	sr := &structs.AllocStopRequest{
 		AllocID:         allocID,
-		NoShutdownDelay: noShutdownDelay,
+		NoShutdownDelay: *noShutdownDelay,
+		Reschedule:      *reschedule,
 	}
 	s.parseWriteRequest(req, &sr.WriteRequest)
 
