@@ -13,6 +13,7 @@ import (
 	"path/filepath"
 	"regexp"
 	"runtime"
+	"slices"
 	"strings"
 	"time"
 
@@ -90,6 +91,8 @@ var (
 	// a taskConfig within a job. It is returned in the TaskConfigSchema RPC
 	taskConfigSpec = hclspec.NewObject(map[string]*hclspec.Spec{
 		"image_path":        hclspec.NewAttr("image_path", "string", true),
+		"emulator":          hclspec.NewAttr("emulator", "string", false),
+		"machine_type":      hclspec.NewAttr("machine_type", "string", false),
 		"drive_interface":   hclspec.NewAttr("drive_interface", "string", false),
 		"accelerator":       hclspec.NewAttr("accelerator", "string", false),
 		"graceful_shutdown": hclspec.NewAttr("graceful_shutdown", "bool", false),
@@ -117,6 +120,8 @@ var (
 // TaskConfig is the driver configuration of a taskConfig within a job
 type TaskConfig struct {
 	ImagePath        string             `codec:"image_path"`
+	Emulator         string             `codec:"emulator"`
+	MachineType      string             `codec:"machine_type"`
 	Accelerator      string             `codec:"accelerator"`
 	Args             []string           `codec:"args"`     // extra arguments to qemu executable
 	PortMap          hclutils.MapStrInt `codec:"port_map"` // A map of host port and the port name defined in the image manifest file
@@ -144,6 +149,10 @@ type Config struct {
 	// include in arguments to qemu, so that cluster operators can can
 	// prevent access to devices
 	ArgsAllowList []string `codec:"args_allowlist"`
+
+	// FingerprintEmulator specifies which QEMU binary is used
+	// for fingerprinting
+	FingerprintEmulator string `codec:"fingerprint_emulator"`
 }
 
 // Driver is a driver for running images via Qemu
@@ -238,13 +247,11 @@ func (d *Driver) buildFingerprint() *drivers.Fingerprint {
 		HealthDescription: drivers.DriverHealthy,
 	}
 
-	bin := "qemu-system-x86_64"
-	if runtime.GOOS == "windows" {
-		// On windows, the "qemu-system-x86_64" command does not respond to the
-		// version flag.
-		bin = "qemu-img"
+	fpEmulator := "qemu-system-x86_64"
+	if d.config.FingerprintEmulator != "" {
+		fpEmulator = d.config.FingerprintEmulator
 	}
-	outBytes, err := exec.Command(bin, "--version").Output()
+	outBytes, err := exec.Command(fpEmulator, "--version").Output()
 	if err != nil {
 		// return no error, as it isn't an error to not find qemu, it just means we
 		// can't use it.
@@ -429,10 +436,19 @@ func (d *Driver) StartTask(cfg *drivers.TaskConfig) (*drivers.TaskHandle, *drive
 
 	// Parse configuration arguments
 	// Create the base arguments
+	emulator := "qemu-system-x86_64"
+	if driverConfig.Emulator != "" {
+		emulator = driverConfig.Emulator
+
+	}
 	accelerator := "tcg"
 	if driverConfig.Accelerator != "" {
 		accelerator = driverConfig.Accelerator
 	}
+	machineType := "pc"
+	if driverConfig.MachineType != "" {
+		machineType = driverConfig.MachineType
+	}
 
 	mb := cfg.Resources.NomadResources.Memory.MemoryMB
 	if mb < 128 || mb > 4000000 {
@@ -440,7 +456,7 @@ func (d *Driver) StartTask(cfg *drivers.TaskConfig) (*drivers.TaskHandle, *drive
 	}
 	mem := fmt.Sprintf("%dM", mb)
 
-	absPath, err := GetAbsolutePath("qemu-system-x86_64")
+	absPath, err := GetAbsolutePath(emulator)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -455,10 +471,11 @@ func (d *Driver) StartTask(cfg *drivers.TaskConfig) (*drivers.TaskHandle, *drive
 
 	args := []string{
 		absPath,
-		"-machine", "type=pc,accel=" + accelerator,
+		"-machine", "type=" + machineType + ",accel=" + accelerator,
 		"-name", vmID,
 		"-m", mem,
-		"-drive", "file=" + vmPath + ",if=" + driveInterface,
+		// setting a drive ID allows users to attach this to other devices
+		"-drive", "file=" + vmPath + ",if=" + driveInterface + ",id=image0",
 		"-nographic",
 	}
 
@@ -545,15 +562,14 @@ func (d *Driver) StartTask(cfg *drivers.TaskConfig) (*drivers.TaskHandle, *drive
 
 	// If using KVM, add optimization args
 	if accelerator == "kvm" {
-		if runtime.GOOS == "windows" {
-			return nil, nil, errors.New("KVM accelerator is unsupported on the Windows platform")
+		if runtime.GOOS == "windows" || runtime.GOOS == "darwin" {
+			return nil, nil, errors.New("KVM accelerator is unsupported on the current platform")
 		}
-		args = append(args,
-			"-enable-kvm",
-			"-cpu", "host",
-		)
 
-		if cfg.Resources.LinuxResources != nil && cfg.Resources.LinuxResources.CpusetCpus != "" {
+		args = append(args, "-enable-kvm")
+
+		// If the user has not set the -smp flag, default to resources.cores
+		if !slices.Contains(args, "-smp") && cfg.Resources.LinuxResources != nil && cfg.Resources.LinuxResources.CpusetCpus != "" {
 			cores := strings.Split(cfg.Resources.LinuxResources.CpusetCpus, ",")
 			args = append(args,
 				"-smp", fmt.Sprintf("%d", len(cores)),
