@@ -163,7 +163,7 @@ func (v *Variable) Value() (cty.Value, hcl.Diagnostics) {
 			Severity: hcl.DiagError,
 			Summary:  fmt.Sprintf("Unset variable %q", v.Name),
 			Detail: "A used variable must be set or have a default value; see " +
-				"https://developer.hashicorp.com/nomad/docs/job-specification/hcl2/variables for " +
+				"https://developer.hashicorp.com/nomad/docs/reference/hcl2/variables  for " +
 				"details.",
 			Context: v.Range.Ptr(),
 		}}
