@@ -26,6 +26,7 @@ import (
 	"github.com/hashicorp/nomad/nomad/state/paginator"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/scheduler"
+	sstructs "github.com/hashicorp/nomad/scheduler/structs"
 )
 
 const (
@@ -285,19 +286,6 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 		}
 	}
 
-	// Preserve the existing task group counts, if so requested
-	if existingJob != nil && args.PreserveCounts {
-		prevCounts := make(map[string]int)
-		for _, tg := range existingJob.TaskGroups {
-			prevCounts[tg.Name] = tg.Count
-		}
-		for _, tg := range args.Job.TaskGroups {
-			if count, ok := prevCounts[tg.Name]; ok {
-				tg.Count = count
-			}
-		}
-	}
-
 	// Submit a multiregion job to other regions (enterprise only).
 	// The job will have its region interpolated.
 	var newVersion uint64
@@ -1052,8 +1040,19 @@ func (j *Job) Scale(args *structs.JobScaleRequest, reply *structs.JobRegisterRes
 			}
 		}
 
+		// Ensure that JobMaxCount is respected.
+		newCount := int(*args.Count)
+		totalCount := 0
+		for _, tg := range job.TaskGroups {
+			totalCount += tg.Count
+		}
+		totalCount = totalCount - group.Count + newCount
+		if j.srv.config.JobMaxCount > 0 && totalCount > j.srv.config.JobMaxCount {
+			return fmt.Errorf("total count was greater than configured job_max_count: %d > %d", totalCount, j.srv.config.JobMaxCount)
+		}
+
 		// Update group count
-		group.Count = int(*args.Count)
+		group.Count = newCount
 		job.SubmitTime = now
 
 		// Block scaling event if there's an active deployment
@@ -1893,7 +1892,7 @@ func (j *Job) Plan(args *structs.JobPlanRequest, reply *structs.JobPlanResponse)
 
 	// Create an in-memory Planner that returns no errors and stores the
 	// submitted plan and created evals.
-	planner := &scheduler.Harness{
+	planner := &sstructs.PlanBuilder{
 		State: &snap.StateStore,
 	}
 
