@@ -16,6 +16,7 @@ const (
 	tableIndex = "index"
 
 	TableNamespaces               = "namespaces"
+	TableNodes                    = "nodes"
 	TableNodePools                = "node_pools"
 	TableServiceRegistrations     = "service_registrations"
 	TableVariables                = "variables"
@@ -83,7 +84,6 @@ func init() {
 		periodicLaunchTableSchema,
 		evalTableSchema,
 		allocTableSchema,
-		vaultAccessorTableSchema,
 		aclPolicyTableSchema,
 		aclTokenTableSchema,
 		oneTimeTokenTableSchema,
@@ -147,7 +147,7 @@ func indexTableSchema() *memdb.TableSchema {
 // This table is used to store all the client nodes that are registered.
 func nodeTableSchema() *memdb.TableSchema {
 	return &memdb.TableSchema{
-		Name: "nodes",
+		Name: TableNodes,
 		Indexes: map[string]*memdb.IndexSchema{
 			// Primary index is used for node management
 			// and simple direct lookup. ID is required to be
@@ -176,6 +176,14 @@ func nodeTableSchema() *memdb.TableSchema {
 					Field: "NodePool",
 				},
 			},
+			indexSigningKey: {
+				Name:         indexSigningKey,
+				AllowMissing: true,
+				Unique:       false,
+				Indexer: &memdb.StringFieldIndex{
+					Field: "IdentitySigningKeyID",
+				},
+			},
 		},
 	}
 }
@@ -825,44 +833,6 @@ func allocTableSchema() *memdb.TableSchema {
 	}
 }
 
-// vaultAccessorTableSchema returns the MemDB schema for the Vault Accessor
-// Table. This table tracks Vault accessors for tokens created on behalf of
-// allocations required Vault tokens.
-func vaultAccessorTableSchema() *memdb.TableSchema {
-	return &memdb.TableSchema{
-		Name: "vault_accessors",
-		Indexes: map[string]*memdb.IndexSchema{
-			// The primary index is the accessor id
-			"id": {
-				Name:         "id",
-				AllowMissing: false,
-				Unique:       true,
-				Indexer: &memdb.StringFieldIndex{
-					Field: "Accessor",
-				},
-			},
-
-			"alloc_id": {
-				Name:         "alloc_id",
-				AllowMissing: false,
-				Unique:       false,
-				Indexer: &memdb.StringFieldIndex{
-					Field: "AllocID",
-				},
-			},
-
-			indexNodeID: {
-				Name:         indexNodeID,
-				AllowMissing: false,
-				Unique:       false,
-				Indexer: &memdb.StringFieldIndex{
-					Field: "NodeID",
-				},
-			},
-		},
-	}
-}
-
 // aclPolicyTableSchema returns the MemDB schema for the policy table.
 // This table is used to store the policies which are referenced by tokens
 func aclPolicyTableSchema() *memdb.TableSchema {
