@@ -623,7 +623,7 @@ func (w *Worker) invokeScheduler(snap *state.StateSnapshot, eval *structs.Evalua
 	// Create the scheduler, or use the special core scheduler
 	var sched sstructs.Scheduler
 	if eval.Type == structs.JobTypeCore {
-		sched = NewCoreScheduler(w.srv, snap)
+		sched = NewCoreScheduler(w.srv, snap, w)
 	} else {
 		sched, err = scheduler.NewScheduler(eval.Type, w.logger, w.srv.workersEventCh, snap, w)
 		if err != nil {
@@ -664,14 +664,7 @@ func (w *Worker) SubmitPlan(plan *structs.Plan) (*structs.PlanResult, sstructs.S
 	plan.SnapshotIndex = w.snapshotIndex
 
 	// Normalize stopped and preempted allocs before RPC
-	normalizePlan := w.srv.peersCache.ServersMeetMinimumVersion(
-		w.srv.Region(),
-		MinVersionPlanNormalization,
-		true,
-	)
-	if normalizePlan {
-		plan.NormalizeAllocations()
-	}
+	plan.NormalizeAllocations()
 
 	// Setup the request
 	req := structs.PlanRequest{
