@@ -707,10 +707,8 @@ func (s *GenericScheduler) computePlacements(
 				// blocked eval without dropping the reschedule tracker
 				if prevAllocation != nil {
 					if missing.IsRescheduling() {
-						updatedPrevAllocation := prevAllocation.Copy()
 						missing.SetPreviousAllocation(prevAllocation)
-						annotateRescheduleTracker(updatedPrevAllocation, structs.LastRescheduleFailedToPlace)
-						swapAllocInPlan(s.plan, prevAllocation, updatedPrevAllocation)
+						markFailedToReschedule(s.plan, prevAllocation, s.job)
 					}
 				}
 
@@ -722,6 +720,25 @@ func (s *GenericScheduler) computePlacements(
 	return nil
 }
 
+// markFailedToReschedule takes a "previous" allocation that we were unable to
+// reschedule and updates the plan to annotate its reschedule tracker and to
+// move it out of the stop list and into the update list so that we don't drop
+// tracking information in the plan applier
+func markFailedToReschedule(plan *structs.Plan, original *structs.Allocation, job *structs.Job) {
+	updated := original.Copy()
+	annotateRescheduleTracker(updated, structs.LastRescheduleFailedToPlace)
+
+	plan.PopUpdate(original)
+	nodeID := original.NodeID
+	for i, alloc := range plan.NodeAllocation[nodeID] {
+		if alloc.ID == original.ID {
+			plan.NodeAllocation[nodeID][i] = updated
+			return
+		}
+	}
+	plan.AppendAlloc(updated, job)
+}
+
 // swapAllocInPlan updates a plan to swap out an allocation that's already in
 // the plan with an updated definition of that allocation. The updated
 // definition should be a deep copy.
